from __future__ import print_function
import sys

from podsixnet2.asyncwrapper import asynchat
from podsixnet2.rencode import loads, dumps

class Channel(asynchat.async_chat):
    endchars = '\0---\0'
    def __init__(self, conn=None, addr=(), server=None, map=None):
        asynchat.async_chat.__init__(self, getattr(conn, "socket", conn), map)
        self.addr = addr
        self._server = server
        self._ibuffer = b""
        self.set_terminator(self.endchars.encode())
        self.sendqueue = []
    
    def collect_incoming_data(self, data):
        self._ibuffer += data
    
    def found_terminator(self):
        data = loads(self._ibuffer)
        self._ibuffer = b""
        
        if type(dict()) == type(data) and 'action' in data:
            [getattr(self, n)(data) for n in ('Network_' + data['action'], 'Network') if hasattr(self, n)]
        else:
            print("OOB data:", data)
    
    def Pump(self):
        [asynchat.async_chat.push(self, d) for d in self.sendqueue]
        self.sendqueue = []
    
    def Send(self, data):
        """Returns the number of bytes sent after enoding."""
        outgoing = dumps(data) + self.endchars.encode()
        self.sendqueue.append(outgoing)
        return len(outgoing)
    
    def handle_connect(self):
        if hasattr(self, "Connected"):
            self.Connected()
        else:
            print("Unhandled Connected()")
    
    def handle_error(self):
        try:
            self.close()
        except:
            pass
        if hasattr(self, "Error"):
            self.Error(sys.exc_info()[1])
        else:
            asynchat.async_chat.handle_error(self)
    
    def handle_expt(self):
        pass
    
    def handle_close(self):
        if hasattr(self, "Close"):
            self.Close()
        asynchat.async_chat.handle_close(self)



from __future__ import print_function
import socket

from podsixnet2.asyncwrapper import poll, asyncore
from podsixnet2.Channel import Channel

class Server(asyncore.dispatcher):
    channelClass = Channel
    
    def __init__(self, channelClass=None, localaddr=("127.0.0.1", 5071), listeners=5):
        if channelClass:
            self.channelClass = channelClass
        self._map = {}
        self.channels = []
        asyncore.dispatcher.__init__(self, map=self._map)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        self.set_reuse_addr()
        self.bind(localaddr)
        self.listen(listeners)
    
    def handle_accept(self):
        try:
            conn, addr = self.accept()
        except socket.error:
            print('warning: server accept() threw an exception')
            return
        except TypeError:
            print('warning: server accept() threw EWOULDBLOCK')
            return
        print("connection")
        self.channels.append(self.channelClass(conn, addr, self, self._map))
        self.channels[-1].Send({"action": "connected"})
        if hasattr(self, "Connected"):
            self.Connected(self.channels[-1], addr)
    
    def Pump(self):
        [c.Pump() for c in self.channels]
        poll(map=self._map)




# coding=utf-8
import socket

from podsixnet2.asyncwrapper import poll
from podsixnet2.Channel import Channel

class EndPoint(Channel):
    """
    The endpoint queues up all network events for other classes to read.
    """
    def __init__(self, address=("127.0.0.1", 31425), map=None):
        self.address = address
        self.isConnected = False
        self.queue = []
        if map is None:
            self._map = {}
        else:
            self._map = map
    
    def DoConnect(self, address=None):
        if address:
            self.address = address
        try:
            Channel.__init__(self, map=self._map)
            self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
            self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            self.connect(self.address)
        except socket.gaierror as e:
            self.queue.append({"action": "error", "error": e.args})
        except socket.error as e:
            self.queue.append({"action": "error", "error": e.args})
    
    def GetQueue(self):
        return self.queue
    
    def Pump(self):
        Channel.Pump(self)
        self.queue = []
        poll(map=self._map)
    
    # methods to add network data to the queue depending on network events
    
    def Close(self):
        self.isConnected = False
        self.close()
        self.queue.append({"action": "disconnected"})
    
    def Connected(self):
        self.queue.append({"action": "socketConnect"})
    
    def Network_connected(self, data):
        self.isConnected = True
    
    def Network(self, data):
        self.queue.append(data)
    
    def Error(self, error):
        self.queue.append({"action": "error", "error": error})
    
    def ConnectionError(self):
        self.isConnected = False
        self.queue.append({"action": "error", "error": (-1, "Connection error")})
 



"""
A client's connection to the server.

This module contains two components: a singleton called 'connection' and a class called 'ConnectionListener'.

'connection' is a singleton instantiation of an EndPoint which will be connected to the server at the other end. It's a singleton because each client should only need one of these in most multiplayer scenarios. (If a client needs more than one connection to the server, a more complex architecture can be built out of instantiated EndPoint()s.) The connection is based on Python's asyncore and so it should have it's polling loop run periodically, probably once per gameloop. This just means putting "from Connection import connection; connection.Pump()" somewhere in your top level gameloop.

Subclass ConnectionListener in order to have an object that will receive network events. For example, you might have a GUI element which is a label saying how many players there are online. You would declare it like 'class NumPlayersLabel(ConnectionListener, ...):' Later you'd instantitate it 'n = NumPlayersLabel()' and then somewhere in your loop you'd have 'n.Pump()' which asks the connection singleton if there are any new messages from the network, and calls the 'Network_' callbacks for each bit of new data from the server. So you'd implement a method like "def Network_players(self, data):" which would be called whenever a message from the server arrived which looked like {"action": "players", "number": 5}.
"""

from __future__ import print_function

from podsixnet2.EndPoint import EndPoint

connection = EndPoint()

class ConnectionListener:
    """
    Looks at incoming data and calls "Network_" methods in self, based on what messages come in.
    Subclass this to have your own classes monitor incoming network messages.
    For example, a method called "Network_players(self, data)" will be called when a message arrives like:
        {"action": "players", "number": 5, ....}
    """
    def Connect(self, *args, **kwargs):
        connection.DoConnect(*args, **kwargs)
        # check for connection errors:
        self.Pump()
    
    def Pump(self):
        for data in connection.GetQueue():
            [getattr(self, n)(data) for n in ("Network_" + data['action'], "Network") if hasattr(self, n)]
                
    def Send(self, data):
        """ Convenience method to allow this listener to appear to send network data, whilst actually using connection. """
        connection.Send(data)

if __name__ == "__main__":
    from time import sleep
    from sys import exit
    class ConnectionTest(ConnectionListener):
        def Network(self, data):
            print("Network:", data)
        
        def Network_error(self, error):
            print("error:", error['error'])
            print("Did you start a server?")
            exit(-1)
        
        def Network_connected(self, data):
            print("connection test Connected")
    
    c = ConnectionTest()
    
    c.Connect()
    while 1:
        connection.Pump()
        c.Pump()
        sleep(0.001)



# coding=utf-8
from __future__ import print_function

import unittest
import sys
from time import sleep, time
import socket

from podsixnet2.asyncwrapper import poll, asyncore
from podsixnet2.Server import Server
from podsixnet2.Channel import Channel
from podsixnet2.EndPoint import EndPoint

class FailEndPointTestCase(unittest.TestCase):
    def setUp(self):
        
        class FailEndPoint(EndPoint):
            def __init__(self):
                EndPoint.__init__(self, ("localhost", 31429))
                self.result = ""
            
            def Error(self, error):
                self.result = error
            
            def Test(self):
                self.DoConnect()
                start = time()
                while not self.result and time() - start < 10:
                    self.Pump()
                    sleep(0.001)
        
        self.endpoint_bad = FailEndPoint()
    
    def runTest(self):
        self.endpoint_bad.Test()
        want = "[Errno 111] Connection refused"
        self.assertEqual(str(self.endpoint_bad.result), str(want), "Socket got %s instead of %s" % (str(self.endpoint_bad.result), str(want)))
    
    def tearDown(self):
        del self.endpoint_bad

class EndPointTestCase(unittest.TestCase):
    def setUp(self):
        self.outgoing = [
            {"action": "hello", "data": {"a": 321, "b": [2, 3, 4], "c": ["afw", "wafF", "aa", "weEEW", "w234r"], "d": ["x"] * 256}},
            {"action": "hello", "data": [454, 35, 43, 543, "aabv"]},
            {"action": "hello", "data": [10] * 512},
            #{"action": "hello", "data": [10] * 512, "otherstuff": "hello\0---\0goodbye", "x": [0, "---", 0], "y": "zÃ¤Ã¶"},
        ]
        self.count = len(self.outgoing)
        self.lengths = [len(data['data']) for data in self.outgoing]
        
        class ServerChannel(Channel):
            def Network_hello(self, data):
                self._server.received.append(data)
                self._server.count += 1
                self.Send({"action": "gotit", "data": "Yeah, we got it: " + str(len(data['data'])) + " elements"})
        
        class TestEndPoint(EndPoint):
            received = []
            connected = False
            count = 0
            
            def Network_connected(self, data):
                self.connected = True
            
            def Network_gotit(self, data):
                self.received.append(data)
                self.count += 1
                
        
        class TestServer(Server):
            connected = False
            received = []
            count = 0
            
            def Connected(self, channel, addr):
                self.connected = True
        
        self.server = TestServer(channelClass=ServerChannel, localaddr=("127.0.0.1", 31426))
        self.endpoint = TestEndPoint(("127.0.0.1", 31426))
    
    def runTest(self):
        self.endpoint.DoConnect()
        for o in self.outgoing:
            self.endpoint.Send(o)
        
        
        for x in range(50):
            self.server.Pump()
            self.endpoint.Pump()
            
            # see if what we receive from the server is what we expect
            for r in self.server.received:
                self.assertTrue(r == self.outgoing.pop(0))
            self.server.received = []
            
            # see if what we receive from the client is what we expect
            for r in self.endpoint.received:
                self.assertTrue(r['data'] == "Yeah, we got it: %d elements" % self.lengths.pop(0))
            self.endpoint.received = []
            
            sleep(0.001)
        
        self.assertTrue(self.server.connected, "Server is not connected")
        self.assertTrue(self.endpoint.connected, "Endpoint is not connected")
        
        self.assertTrue(self.server.count == self.count, "Didn't receive the right number of messages")
        self.assertTrue(self.endpoint.count == self.count, "Didn't receive the right number of messages")
        
        self.endpoint.Close()
        
    
    def tearDown(self):
        del self.server
        del self.endpoint

class ServerTestCase(unittest.TestCase):
    testdata = {"action": "hello", "data": {"a": 321, "b": [2, 3, 4], "c": ["afw", "wafF", "aa", "weEEW", "w234r"], "d": ["x"] * 256}}
    def setUp(self):
        print("ServerTestCase")
        print("--------------")
        
        class ServerChannel(Channel):
            def Network_hello(self, data):
                print("*Server* ran test method for 'hello' action")
                print("*Server* received:", data)
                self._server.received = data
        
        class EndPointChannel(Channel):
            connected = False
            def Connected(self):
                print("*EndPoint* Connected()")
            
            def Network_connected(self, data):
                self.connected = True
                print("*EndPoint* Network_connected(", data, ")")
                print("*EndPoint* initiating send")
                self.Send(ServerTestCase.testdata)
        
        class TestServer(Server):
            connected = False
            received = None
            def Connected(self, channel, addr):
                self.connected = True
                print("*Server* Connected() ", channel, "connected on", addr)
        
        self.server = TestServer(channelClass=ServerChannel, localaddr=("127.0.0.1", 31427))
        
        sender = asyncore.dispatcher(map=self.server._map)
        sender.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        sender.connect(("127.0.0.1", 31427))
        self.outgoing = EndPointChannel(sender, map=self.server._map)
        
    def runTest(self):
        from time import sleep
        print("*** polling for half a second")
        for x in range(250):
            self.server.Pump()
            self.outgoing.Pump()
            if self.server.received:
                self.assertTrue(self.server.received == self.testdata)
                self.server.received = None
            sleep(0.001)
        self.assertTrue(self.server.connected == True, "Server is not connected")
        self.assertTrue(self.outgoing.connected == True, "Outgoing socket is not connected")
    
    def tearDown(self):
        pass
        del self.server
        del self.outgoing


if __name__ == "__main__":
    unittest.main()




#!/usr/bin/env python

import os
from distutils.core import setup
import subprocess

# convert readme to rst format
try:
   import pypandoc
   long_description = pypandoc.convert('README.md', 'rst')
except:
   long_description = ''

versionfile = "podsixnet/version.py"
if not os.path.isfile(versionfile):
    # assume git checkout
    __version__ = subprocess.check_output(["git", "describe", "--tag", "--always"]).decode("utf8").strip("\n")
else:
    # created by pip
    exec(open(versionfile).read())

setup(
    version=__version__,
    name='podsixnet2',
    description='Multiplayer networking library for games',
    long_description=long_description,
    author='Chris McCormick',
    author_email='chris@mccormick.cx',
    url='https://github.com/chr15m/PodSixNet',
    packages=['podsixnet2'],
)
